--*****************************************************************************
--*  Copyright (c) 2015 by Michael Fischer (www.emb4fun.de)
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  20.07.2015  mifi  First version.
--*  07.11.2015  mifi  Tested with Wishbone and Avalon Master.
--*****************************************************************************

--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity wb_gpio_if is
   port (  
      -- Syscon
      clk_i    : in  std_logic := '0';
      rst_i    : in  std_logic := '0';
      
      -- Wishbone
      wb_stb_i : in  std_logic := '0';
      wb_cyc_i : in  std_logic := '0';      -- Not used here
      wb_we_i  : in  std_logic := '0';
      wb_ack_o : out std_logic;
      wb_adr_i : in  std_logic_vector(2 downto 0)  := (others => '0');
      wb_dat_i : in  std_logic_vector(15 downto 0) := (others => '0');
      wb_dat_o : out std_logic_vector(15 downto 0);  
            
      -- None Wishbone
      port_i   : in  std_logic_vector(15 downto 0) := (others => '0');
      port_o   : out std_logic_vector(15 downto 0)
   );
end entity wb_gpio_if;


--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of wb_gpio_if is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   component gpio_reg is
	   port (
		   clk_i       : in  std_logic                     := '0';             --   clock.clk
		   reset_i     : in  std_logic                     := '0';             --   reset.reset
      
		   address_i   : in  std_logic_vector(2 downto 0)  := (others => '0'); --      s0.address
		   read_i      : in  std_logic                     := '0';             --        .read
		   readdata_o  : out std_logic_vector(15 downto 0);                    --        .readdata
		   write_i     : in  std_logic                     := '0';             --        .write
		   writedata_i : in  std_logic_vector(15 downto 0) := (others => '0'); --        .writedata
      
         wb_ack_o    : out std_logic;
      
		   port_o      : out std_logic_vector(15 downto 0);                    -- conduit.export
		   port_i      : in  std_logic_vector(15 downto 0) := (others => '0')  --        .export
	   );
   end component gpio_reg;   
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------
   
   signal read  : std_logic; 
   signal write : std_logic;
   signal ack   : std_logic;
   
begin

   -- Some simplifications for later use
   read  <= '1' when ((wb_stb_i = '1') and (wb_we_i = '0')) else '0';
   write <= '1' when ((wb_stb_i = '1') and (wb_we_i = '1')) else '0';

   inst_gpio : gpio_reg 
      port map ( 
         clk_i       => clk_i,
         reset_i     => rst_i,

		   address_i   => wb_adr_i,
		   read_i      => read,
		   readdata_o  => wb_dat_o,
		   write_i     => write,
		   writedata_i => wb_dat_i,
         
         wb_ack_o    => wb_ack_o,
      
		   port_o      => port_o,
		   port_i      => port_i
      );
   
end architecture syn;

-- *** EOF ***

